#!/usr/bin/env perl

# Some tests for 'darcs add'

use Test::More tests => 13;
use strict;
use vars qw/$DARCS/;

die 'darcs not found' unless $ENV{DARCS} || (-x "$ENV{PWD}/../darcs");
$DARCS = $ENV{DARCS} || "$ENV{PWD}/../darcs";

`rm -rf temp1`;
`mkdir temp1`;
 chdir 'temp1';
`$DARCS init`;

###

my $test_name = 'Make sure that messages about directories call them directories.';
`mkdir foo.d`;
`$DARCS add foo.d`;
# Try adding the same directory when it's already in the repo 
like(`$DARCS add foo.d`, qr/directory/,$test_name);

###

$test_name = 'Check unusual filename handling';
my $filename = "\03.txt";

open(STUPIDFILE, ">$filename");
print STUPIDFILE "foobar\n";
close(STUPIDFILE);
my $result = `$DARCS add $filename`;
like($result,qr/^$/, $test_name);

like(`$DARCS whatsnew`,qr/hunk \.\/.*\\03.*\.txt 1/,
     "whatsnew with funky filename");

$test_name = 'Check UTF filename handling';
$filename = chr(0x351).'utf.txt';

open(STUPIDFILE, ">$filename");
print STUPIDFILE "Foo Test\n";
close(STUPIDFILE);
my $result = `$DARCS add $filename`;
like($result,qr/^$/, $test_name);

like(`$DARCS whatsnew`,qr/addfile.*\\c3.*\\8d.*\\c2.*\\91.*utf.txt/,
     "whatsnew shows addfile with UTF8 filename");

like(`$DARCS whatsnew`,qr/hunk.*\\c3.*\\8d.*\\c2.*\\91.*utf.txt/,
     "whatsnew shows hunk with UTF8 filename");

like(`$DARCS record --all -A me -m foo`,qr/^Finished recording/,
     "recording with UTF8 filename");
is($?,0, " return code == 0");

$test_name = 'Check filename handling with whitespace';
$filename = "space tab\tnewline\n.txt";

open(STUPIDFILE, ">$filename");
print STUPIDFILE "Foo Test\n";
close(STUPIDFILE);
my $result = `$DARCS add '$filename'`;
like($result,qr/^$/, $test_name);

like(`$DARCS whatsnew`, qr/space\\32\\tab\\9\\newline\\10\\.txt/,
     "whatsnew with whitespace in filename");
like(`$DARCS record --all -A me -m foo`,qr/^Finished recording/,
     "recording with UTF8 filename");
is($?,0, " return code == 0");

#chdir '../';
#`rm -rf temp1`;
ok((!-d 'temp1'), 'temp1 directory was deleted');




