#!/usr/bin/env perl

# Some tests for 'darcs changes'

use Test::More qw/no_plan/;
use strict;
use vars qw/$DARCS/;

die 'darcs not found' unless $ENV{DARCS} || (-x "$ENV{PWD}/../darcs");
$DARCS = $ENV{DARCS} || "$ENV{PWD}/../darcs";

`rm -rf temp1`;
`mkdir temp1`;
chdir 'temp1';
`$DARCS init`;

###

my $test_name = 'record something';

`date >> date.t`;
`darcs add date.t`;

like(`darcs record -A x -a -m foo date.t 2>&1`, qr/finished recording/i, $test_name);

####

like(`darcs changes date.t`,qr/foo/,'changes file.txt: trivial case works');
like(`darcs changes --last=1 date.t`,qr/foo/,'changes --last=1 file.txt');
like(`darcs changes --last=1 --summary date.t`,qr/foo/,'changes --last=1 --summary file.txt');

###

# Add 50 records and try again 
for (my $i = 0; $i <= 49; $i++) {
    `date >> date.t`;
    `darcs record -A x -a -m "foo record num $i" date.t 2>&1`;
}

like(`darcs changes date.t`,qr/foo/,'after 50 records: changes file.txt: trivial case works');
like(`darcs changes --last=1 date.t`,qr/foo/,'after 50 records: changes --last=1 file.txt');
like(`darcs changes --last=1 --summary date.t`,qr/foo/,'after 50 records: changes --last=1 --summary file.txt');


chdir '../';
`rm -rf temp1`;
ok((!-d 'temp1'), 'temp1 directory was deleted');




